<?php
/**
 * Eduline Footer Settings panel at Theme Customizer
 *
* @package Eduline_Pro
 * @since 1.0.0
 */

add_action( 'customize_register', 'eduline_footer_settings_register' );

function eduline_footer_settings_register( $wp_customize ) {
  require get_template_directory() .'/inc/repeater/class-repeater-settings.php';
  require get_template_directory() .'/inc/repeater/class-control-repeater.php';

	/**
     * Add Footer Settings Panel
     *
     * @since 1.0.0
     */
  $wp_customize->add_panel(
   'eduline_footer_settings_panel',
   array(
     'priority'       => 30,
     'capability'     => 'edit_theme_options',
     'theme_supports' => '',
     'title'          => __( 'Footer Settings', 'eduline-pro' ),
   )
 );


  /*----------------------------------------------------------------------------------------------------------------------------------------*/
  /**
     * Footer Social section
     *
     * @since 1.0.0
     */
  $wp_customize->add_section(
    'footer_social_media_section',
    array(
      'priority'       => 2,
      'panel'          => 'eduline_footer_settings_panel',
      'capability'     => 'edit_theme_options',
      'theme_supports' => '',
      'title'          => __( 'Footer Social Media Section', 'eduline-pro' ),
      'description'    => __( 'Managed the content display at Footer Social media section.', 'eduline-pro' ),
    )
  );

  /**Foter Social medi  Enable/Disable  */
  $wp_customize->add_setting(
    'footer_social_media_enable',
    array(
      'default'           => 0,
      'sanitize_callback' => 'eduline_sanitize_checkbox',
    )
  );

  $wp_customize->add_control(
    'footer_social_media_enable',
    array(
      'section'     => 'footer_social_media_section',
      'label'       => __( 'Social Links', 'eduline-pro' ),
      'description' => __( 'Enable/Disable social links in Footer.', 'eduline-pro' ),
      'type'        => 'checkbox'
    )       
  );

  /** Social Links */
  $wp_customize->add_setting( 
    new Eduline_Repeater_Setting( 
      $wp_customize, 
      'eduline_social_links', 
      array(
        'default' => array(
          array(
            'font' => '',
            'link' => '',                        
          ),
          array(
            'font' => '',
            'link' => '',
          ),
          array(
            'font' => '',
            'link' => '',
          ),
          array(
            'font' => '',
            'link' => '',
          )
        ),
        'sanitize_callback' => array( 'Eduline_Repeater_Setting', 'sanitize_repeater_setting' ),
      ) 
    ) 
  );

  $wp_customize->add_control(
    new Eduline_Control_Repeater(
      $wp_customize,
      'eduline_social_links',
      array(
        'section' => 'footer_social_media_section',              
        'label'   => __( 'Social Links', 'eduline-pro' ),
        'fields'  => array(
          'font' => array(
            'type'        => 'font',
            'label'       => __( 'Font Awesome Icon', 'eduline-pro' ),
            'description' => __( 'Example: fa-facebook', 'eduline-pro' ),
          ),
          'link' => array(
            'type'        => 'url',
            'label'       => __( 'Link', 'eduline-pro' ),
            'description' => __( 'Example: http://facebook.com', 'eduline-pro' ),
          )
        ),
        'row_label' => array(
          'type' => 'field',
          'value' => __( 'social', 'eduline-pro' ),
          'field' => 'link'
        )                        
      )
    )
  );



   /**
     * Footer Copy Right Text section
     *
     * @since 1.0.0
     */
   $wp_customize->add_section(
    'footer_copyright_section',
    array(
      'priority'       => 2,
      'panel'          => 'eduline_footer_settings_panel',
      'capability'     => 'edit_theme_options',
      'theme_supports' => '',
      'title'          => __( 'Footer Copy Right Section', 'eduline-pro' ),
      'description'    => __( 'Managed the content display at Footer Copy Right section.', 'eduline-pro' ),
    )
  );


   $wp_customize->add_setting('eduline_footer_copyright_text',array(
    'capability'     => 'edit_theme_options',
    'sanitize_callback' => 'sanitize_text_field',
    'default' =>  ''
  )
 );

   $wp_customize->add_control(new WP_Customize_Control($wp_customize,'eduline_footer_copyright_text',array(
    'label' => __('Footer Copyright Text','eduline-pro'),
    'description' => __('Eg:- &copy; All Right Reserved 2018','eduline-pro'),
    'type' => 'text',
    'section' => 'footer_copyright_section',
    'settings' => 'eduline_footer_copyright_text',
  )
 ));
 }