<?php
/**
 * Eduline Pages Settings panel at Theme Customizer
 *
 * @package Eduline_Pro
 * @since 1.0.0
 */

add_action( 'customize_register', 'eduline_page_settings_register' );

function eduline_page_settings_register( $wp_customize ) {
  require get_template_directory() .'/inc/repeater/class-repeater-settings.php';
  require get_template_directory() .'/inc/repeater/class-control-repeater.php';

	/**
     * Add Page Settings Panel
     *
     * @since 1.0.0
     */
  $wp_customize->add_panel(
   'eduline_page_settings_panel',
   array(
     'priority'       => 40,
     'capability'     => 'edit_theme_options',
     'theme_supports' => '',
     'title'          => __( 'Page Template Settings', 'eduline-pro' ),
   )
 );

  /*----------------------------------------------------------------------------------------------------------------------------------------*/
	/**
     * Contact Page section
     *
     * @since 1.0.0
     */
  $wp_customize->add_section(
    'eduline_contact_page_section',
    array(
     'priority'       => 5,
     'panel'          => 'eduline_page_settings_panel',
     'capability'     => 'edit_theme_options',
     'theme_supports' => '',
     'title'          => __( 'Contact Page', 'eduline-pro' ),
     'description'    => __( 'Managed the content display at contact page.', 'eduline-pro' ),
   )
  );

  // Select Page For Contact Page Title and description
  $wp_customize->add_setting( 'eduline_contact_page_title', array(
    'capability'            => 'edit_theme_options',
    'default'               => '',
    'sanitize_callback'     => 'eduline_sanitize_dropdown_pages'
  ) );

  $wp_customize->add_control('eduline_contact_page_title',
    array(
      'label'                 =>  __( 'Select Page for Contact Page  title and description', 'eduline-pro' ),
      'section' => 'eduline_contact_page_section',
      'type'=> 'dropdown-pages',
      'settings' => 'eduline_contact_page_title'
    )
  );

  /**Contact Page Quotes Heading */
  $wp_customize->add_setting( 'eduline_contact_page_quotes_text', array(
    'capability'            => 'edit_theme_options',
    'default'               => '',
    'sanitize_callback'     => 'sanitize_text_field'
  ) );

  $wp_customize->add_control( 'eduline_contact_page_quotes_text', array(
    'label'                 =>  __( 'Type Contact Page Quotes', 'eduline-pro' ),
    'description'           =>__('Eg:-"Build your Bright Future at Edugrade, Get in Touch, minim veniam,We would love to help you! ','eduline-pro'),
    'section'               => 'eduline_contact_page_section',
    'type'                  => 'text',
    'settings'              => 'eduline_contact_page_quotes_text',
  ) );

  /**Contact Page Get In Touch Heading */
  $wp_customize->add_setting( 'eduline_contact_page_git_text', array(
    'capability'            => 'edit_theme_options',
    'default'               => '',
    'sanitize_callback'     => 'sanitize_text_field'
  ) );

  $wp_customize->add_control( 'eduline_contact_page_git_text', array(
    'label'                 =>  __( 'Type Contact Page Quotes', 'eduline-pro' ),
    'description'           =>__('Eg:- Get In Touch','eduline-pro'),
    'section'               => 'eduline_contact_page_section',
    'type'                  => 'text',
    'settings'              => 'eduline_contact_page_git_text',
  ) );

  /**Contact Page Main Address */
  $wp_customize->add_setting( 'eduline_contact_main_address', array(
    'capability'            => 'edit_theme_options',
    'default'               => '',
    'sanitize_callback'     => 'sanitize_text_field'
  ) );

  $wp_customize->add_control( 'eduline_contact_main_address', array(
    'label'                 =>  __( 'Type Main Address', 'eduline-pro' ),
    'description'           =>__('Eg:-  Street # 75, Borne Block, 5th Avenue, West Road, NY.','eduline-pro'),
    'section'               => 'eduline_contact_page_section',
    'type'                  => 'text',
    'settings'              => 'eduline_contact_main_address',
  ) );

  /**Contact Page Phone Number */
  $wp_customize->add_setting( 'eduline_contact_phone_number', array(
    'capability'            => 'edit_theme_options',
    'default'               => '',
    'sanitize_callback'     => 'sanitize_text_field'
  ) );

  $wp_customize->add_control( 'eduline_contact_phone_number', array(
    'label'                 =>  __( 'Type Phone Number', 'eduline-pro' ),
    'description'           =>__('Eg:-  (800) 457 392','eduline-pro'),
    'section'               => 'eduline_contact_page_section',
    'type'                  => 'text',
    'settings'              => 'eduline_contact_phone_number',
  ) );

  /**Contact Page Email Address */
  $wp_customize->add_setting( 'eduline_contact_email_address', array(
    'capability'            => 'edit_theme_options',
    'default'               => '',
    'sanitize_callback'     => 'sanitize_text_field'
  ) );

  $wp_customize->add_control( 'eduline_contact_email_address', array(
    'label'                 =>  __( 'Type Email Address', 'eduline-pro' ),
    'description'           =>__('Eg:-  info.demo@email.com','eduline-pro'),
    'section'               => 'eduline_contact_page_section',
    'type'                  => 'text',
    'settings'              => 'eduline_contact_email_address',
  ) );

  

    // Contact form
  $wp_customize->add_setting( 'eduline_contact_form_option', array(
    'capability'            => 'edit_theme_options',
    'default'               => '',
    'sanitize_callback'     => 'sanitize_text_field'
  ) );

  $wp_customize->add_control( 'eduline_contact_form_option', array(
    'label'                 =>  __( 'Contact Form Section Use Shortcode', 'eduline-pro' ),
    'description'           =>  __( 'eg [contact-form-7 id="108" title="Contact form 1"]', 'eduline-pro' ),
    'section'               => 'eduline_contact_page_section',
    'type'                  => 'text',
    'settings' => 'eduline_contact_form_option',
  ) );


  /** Social Links */
  $wp_customize->add_setting( 
    new Eduline_Repeater_Setting( 
      $wp_customize, 
      'eduline_contact_page_social_items', 
      array(
        'default' => array(
          array(
            'font' => '',
            'link' => '',                        
          ),
          array(
            'font' => '',
            'link' => '',
          ),
          array(
            'font' => '',
            'link' => '',
          ),
          array(
            'font' => '',
            'link' => '',
          )
        ),
        'sanitize_callback' => array( 'Eduline_Repeater_Setting', 'sanitize_repeater_setting' ),
      ) 
    ) 
  );

  $wp_customize->add_control(
    new Eduline_Control_Repeater(
      $wp_customize,
      'eduline_contact_page_social_items',
      array(
        'section' => 'eduline_contact_page_section',              
        'label'   => __( 'Social Links', 'eduline-pro' ),
        'fields'  => array(
          'font' => array(
            'type'        => 'font',
            'label'       => __( 'Font Awesome Icon', 'eduline-pro' ),
            'description' => __( 'Example: fa-facebook', 'eduline-pro' ),
          ),
          'link' => array(
            'type'        => 'url',
            'label'       => __( 'Link', 'eduline-pro' ),
            'description' => __( 'Example: http://facebook.com', 'eduline-pro' ),
          )
        ),
        'row_label' => array(
          'type' => 'field',
          'value' => __( 'social', 'eduline-pro' ),
          'field' => 'link'
        )                        
      )
    )
  );

  // Course Pricing Page Section

  $wp_customize->add_section(
    'eduline_course_pricing_section',
    array(
     'priority'       => 10,
     'panel'          => 'eduline_page_settings_panel',
     'capability'     => 'edit_theme_options',
     'theme_supports' => '',
     'title'          => __( 'Course Pricing Page', 'eduline-pro' ),
     'description'    => __( 'Managed the content display at Course Pricing page.', 'eduline-pro' ),
   )
  );

  // Select Page For Contact Page Title and description
  $wp_customize->add_setting( 'eduline_course_pricing_title', array(
    'capability'            => 'edit_theme_options',
    'default'               => '',
    'sanitize_callback'     => 'eduline_sanitize_dropdown_pages'
  ) );

  $wp_customize->add_control('eduline_course_pricing_title',
    array(
      'label'                 =>  __( 'Select Page for Course Pricing Page  title and description', 'eduline-pro' ),
      'section' => 'eduline_course_pricing_section',
      'type'=> 'dropdown-pages',
      'settings' => 'eduline_course_pricing_title'
    )
  );



  // Setting: Link of Pricing Table Widget.
  class Eduline_Course_Pricing_Widget_Link extends WP_Customize_Control {
    public $type = 'customtext';
    public $extra = ''; // we add this for the extra description
    public function render_content() {
      ?>
      <label>            
        <a href="<?php echo esc_url( 'widgets.php' ); ?>" target='_blank'><?php echo esc_html( $this->label ); ?></a>
        <span><?php echo esc_html( $this->extra ); ?></span>         
      </label>
      <?php
    }
  }


  $wp_customize->add_setting('pricing_table_widget_link', array(
    'default' => '',
    'type' => 'customtext',
    'capability' => 'edit_theme_options',
    'transport' => 'refresh',
    'sanitize_callback' => 'sanitize_text_field',
  ) );
  $wp_customize->add_control( new Eduline_Course_Pricing_Widget_Link( $wp_customize, 'pricing_table_widget_link', array(
    'label' => esc_attr__( 'Go to Widget', 'eduline-pro' ),
    'section' => 'eduline_course_pricing_section',
    'settings' => 'pricing_table_widget_link',
    'extra' => esc_attr__( ' for more info', 'eduline-pro' )
  ) ) 
);
}