<?php
/**
 * Functions which enhance the theme by hooking into WordPress
 *
 * @package Eduline_Pro
 */

/**
 * Adds custom classes to the array of body classes.
 *
 * @param array $classes Classes for the body element.
 * @return array
 */
function eduline_body_classes( $classes ) {
	// Adds a class of hfeed to non-singular pages.
	if ( ! is_singular() ) {
		$classes[] = 'hfeed';
	}

	// Adds a class of no-sidebar when there is no sidebar present.
	if ( ! is_active_sidebar( 'sidebar-1' ) ) {
		$classes[] = 'no-sidebar';
	}

	return $classes;
}
add_filter( 'body_class', 'eduline_body_classes' );

/**
 * Add a pingback url auto-discovery header for single posts, pages, or attachments.
 */
function eduline_pingback_header() {
	if ( is_singular() && pings_open() ) {
		printf( '<link rel="pingback" href="%s">', esc_url( get_bloginfo( 'pingback_url' ) ) );
	}
}
add_action( 'wp_head', 'eduline_pingback_header' );


// Top Header Contact Info
if( ! function_exists('top_header_contact_info_items')):
	function top_header_contact_info_items(){
		$contact_items = get_theme_mod( 'top_header_contact_info_items');
		if( $contact_items  ){ 
			foreach( $contact_items as $contact ){ ?>
				<li><i class="<?php echo esc_attr($contact['icon']);?>"></i><?php echo esc_attr($contact['title']);?></li>
				<?php
			}
		}
	}
endif;	


// Frontpage Counter items
if( ! function_exists( 'eduline_counter_items')):
	function eduline_counter_items(){
		$counter_items = get_theme_mod( 'eduline_counter_items');
		if( $counter_items  ){ 
			foreach( $counter_items as $counter ){ ?>
				<div class="col-lg-3 col-md-3 col-12">
					<!-- Single Fact -->
					<div class="single-fact">
						<i class="<?php echo esc_attr( $counter['icon'] );?>"></i>
						<div class="content">
							<div class="number"><span class="counter"><?php echo esc_attr( $counter['number'] );?></span><?php esc_html_e( 'k+', 'eduline-pro' );?></div>
							<p><?php echo esc_html( $counter['title'] );?></p>
						</div>
					</div>
					<!--/ End Single Fact -->
				</div>
				<?php
			}
		}
	}
endif;


// Footer Social items
if( ! function_exists( 'eduline_footer_social_items') ):
	function eduline_footer_social_items(){
		$defaults =  array(
			array(
				'font' => '',
				'link' => '',                        
			),
			array(
				'font' => '',
				'link' => '',
			),
			array(
				'font' => '',
				'link' => '',
			),
			array(
				'font' => '',
				'link' => '',
			)
		);
		$social_items = get_theme_mod( 'eduline_social_links', $defaults );
		$social_enable = get_theme_mod('footer_social_media_enable');
		if( $social_enable  ){ ?>
			<ul class="social">
				<?php foreach( $social_items as $social ){ ?>
					<li><a href="<?php echo esc_url($social['link']);?>"><i class="<?php echo esc_attr($social['font']);?>"></i></a></li>
					<?php
				}?>
			</ul>
			<?php
		}
	}
endif;


// Contact Page Social items
if( ! function_exists( 'eduline_contact_page_social_items')):
	function eduline_contact_page_social_items(){
		$defaults =  array(
			array(
				'font' => '',
				'link' => '',                        
			),
			array(
				'font' => '',
				'link' => '',
			),
			array(
				'font' => '',
				'link' => '',
			),
			array(
				'font' => '',
				'link' => '',
			)
		);
		$social_items = get_theme_mod( 'eduline_contact_page_social_items', $defaults );
		
		if( $social_items  ){ ?>
			
			<ul class="social">
				<?php foreach( $social_items as $social ){ ?>
					<li><a href="<?php echo esc_url($social['link']);?>"><i class="<?php echo esc_attr($social['font']);?>"></i></a></li>
					<?php
				}?>
			</ul>
			<?php
		}
	}
endif;