<?php
/**
 * Eduline Theme Customizer
 *
 * @package Eduline_Pro
 */

/**
 * Add postMessage support for site title and description for the Theme Customizer.
 *
 * @param WP_Customize_Manager $wp_customize Theme Customizer object.
 */
function eduline_customize_register( $wp_customize ) {
	$wp_customize->get_setting( 'blogname' )->transport         = 'postMessage';
	$wp_customize->get_setting( 'blogdescription' )->transport  = 'postMessage';
	$wp_customize->get_setting( 'header_textcolor' )->transport = 'postMessage';

	if ( isset( $wp_customize->selective_refresh ) ) {
		$wp_customize->selective_refresh->add_partial( 'blogname', array(
			'selector'        => '.site-title a',
			'render_callback' => 'eduline_customize_partial_blogname',
		) );
		$wp_customize->selective_refresh->add_partial( 'blogdescription', array(
			'selector'        => '.site-description',
			'render_callback' => 'eduline_customize_partial_blogdescription',
		) );
	}
}
add_action( 'customize_register', 'eduline_customize_register' );

/**
 * Render the site title for the selective refresh partial.
 *
 * @return void
 */
function eduline_customize_partial_blogname() {
	bloginfo( 'name' );
}

/**
 * Render the site tagline for the selective refresh partial.
 *
 * @return void
 */
function eduline_customize_partial_blogdescription() {
	bloginfo( 'description' );
}

/**
 * Binds JS handlers to make Theme Customizer preview reload changes asynchronously.
 */
function eduline_customize_preview_js() {
	wp_enqueue_script( 'eduline-customizer', get_template_directory_uri() . '/js/customizer.js', array( 'customize-preview' ), '20151215', true );
}
add_action( 'customize_preview_init', 'eduline_customize_preview_js' );


/*----------------------------------------------------------------------------------------------------------------------------------------*/
/**
 * Enqueue required scripts/styles for customizer panel
 *
 * @since 1.0.0
 */
function eduline_customize_backend_scripts() {
	wp_enqueue_script( 'eduline-frontpage-reorgering-controls', get_template_directory_uri() . '/assets/customizer/frontpage-reordering.js' );
	wp_enqueue_style( 'eduline-frontpage-reorgering-controls', get_template_directory_uri() . '/assets/customizer/frontpage-reordering.css' );
}
add_action( 'customize_controls_enqueue_scripts', 'eduline_customize_backend_scripts', 10 );
/*----------------------------------------------------------------------------------------------------------------------------------------*/
/**
 * Load customizer required panels.
 */

require get_template_directory() . '/inc/customizer/general-panel.php';
require get_template_directory() . '/inc/customizer/header-panel.php';
require get_template_directory() . '/inc/customizer/frontpage-panel.php';
require get_template_directory() . '/inc/customizer/blog-panel.php';
require get_template_directory() . '/inc/customizer/page-panel.php';
require get_template_directory() . '/inc/customizer/footer-panel.php';
require get_template_directory() . '/inc/customizer/customizer-css.php';
require get_template_directory() . '/inc/customizer/frontpage-reordering.php';

require get_template_directory() . '/inc/customizer/customizer-sanitize.php';
require get_template_directory() . '/inc/customizer/customizer-classes.php';