<?php

/**
 * Eduline Frontpage Settings panel at Theme Customizer
 *
 * @package Eduline_Pro
 * @since 1.0.0
 */
add_action( 'customize_register', 'eduline_frontpage_settings_register' );

function eduline_frontpage_settings_register( $wp_customize ) {
 require get_template_directory() .'/inc/repeater/class-repeater-settings.php';
 require get_template_directory() .'/inc/repeater/class-control-repeater.php';
/**
 * Add Frontpage Settings Panel
 *
 * @since 1.0.0
 */
$wp_customize->add_panel(
  'eduline_frontpage_settings_panel',
  array(
    'priority'       => 15,
    'capability'     => 'edit_theme_options',
    'theme_supports' => '',
    'title'          => __( 'Frontpage Settings', 'eduline-pro' ),
  )
);

/*----------------------------------------------------------------------------------------------------------------------------------------*/
/**
 * Front page Banner section
 *
 * @since 1.0.0
 */

$wp_customize->add_section(
  'eduline_frontpage_banner_section',
  array(
   'priority'       => 1,
   'panel'          => 'eduline_frontpage_settings_panel',
   'capability'     => 'edit_theme_options',
   'theme_supports' => '',
   'title'          => __( 'Banner Section', 'eduline-pro' ),
   'description'    => __( 'Managed the Banner display at Frontpage section.', 'eduline-pro' ),
 )
);

/**
 * Enable/Disable for Banner
 *
 * @since 1.0.0
 */
$wp_customize->add_setting(
  'eduline_banner_enable',
  array(
    'default'           => 0,
    'sanitize_callback' => 'eduline_sanitize_checkbox',
  )
);

$wp_customize->add_control(
  'eduline_banner_enable',
  array(
    'section'     => 'eduline_frontpage_banner_section',
    'label'       => __( 'Enable/Disable for  Banner', 'eduline-pro' ),
    'type'        => 'checkbox'
  )       
);

/*
* Frontpage banner section
*/
$wp_customize->add_setting(
  'eduline_banner_tag_text',
  array(
    'default'           => __( 'Accelerate your future', 'eduline-pro' ),
    'sanitize_callback' => 'sanitize_text_field',
    'transport'         => 'postMessage'
  )
);

$wp_customize->add_control(
  'eduline_banner_tag_text',
  array(
    'label'    => __( 'Popular Courses Text', 'eduline-pro' ),
    'section'  => 'eduline_frontpage_banner_section',
    'type'     => 'text',
  )
);

$wp_customize->selective_refresh->add_partial( 'eduline_banner_tag_text', array(
  'selector' => '.hero-area  .slider-text .short',
  'render_callback' => 'get_eduline_banner_tag_text'
) );
// Select Page For Banner title with featured image.
$wp_customize->add_setting( 'eduline_banner_title', array(
  'capability'            => 'edit_theme_options',
  'default'               => '',
  'sanitize_callback'     => 'eduline_sanitize_dropdown_pages'
) );

$wp_customize->add_control('eduline_banner_title',
  array(
    'label'                 =>  __( 'Select Page for Banner title with featured image', 'eduline-pro' ),
    'section' => 'eduline_frontpage_banner_section',
    'type'=> 'dropdown-pages',
    'settings' => 'eduline_banner_title'
  )
);

$wp_customize->add_setting( 'eduline_banner_sub_title', array(
  'capability'            => 'edit_theme_options',
  'default'               => '',
  'sanitize_callback'     => 'sanitize_text_field'
) );

$wp_customize->add_control( 'eduline_banner_sub_title', array(
  'label'                 =>  __( 'Sub Title For Banner', 'eduline-pro' ),
  'description'           =>  __( 'Collection', 'eduline-pro' ),
  'section'               => 'eduline_frontpage_banner_section',
  'type'                  => 'text',
  'settings' => 'eduline_banner_sub_title',
) );

/*----------------------------------------------------------------------------------------------------------------------------------------*/
/**
 * Frontpage Counter section
 *
 * @since 1.0.0
 */
$wp_customize->add_section(
  'eduline_counter_section',
  array(
    'priority'       => 2,
    'panel'          => 'eduline_frontpage_settings_panel',
    'capability'     => 'edit_theme_options',
    'theme_supports' => '',
    'title'          => __( 'Counter Section', 'eduline-pro' ),
    'description'    => __( 'Managed the content display at Counter section.', 'eduline-pro' ),
  )
);

/**
* Enable/Disable for Counter
*
* @since 1.0.0
*/
$wp_customize->add_setting(
  'eduline_counter_enable',
  array(
    'default'           => 0,
    'sanitize_callback' => 'eduline_sanitize_checkbox',
  )
);

$wp_customize->add_control(
  'eduline_counter_enable',
  array(
    'section'     => 'eduline_counter_section',
    'label'       => __( 'Enable/Disable for Counter', 'eduline-pro' ),
    'type'        => 'checkbox'
  )       
);


$wp_customize->add_setting( 
  new Eduline_Repeater_Setting( 
    $wp_customize, 
    'eduline_counter_items', 
    array(
      'default' => array(
        array(
          'icon' => '',
          'number'=> '',
          'title' => '',
        ),
        array(
          'icon' => '',
          'number'=> '',
          'title' => '',
        ),
        array(
         'icon' => '',
         'number'=> '',
         'title' => '',
       ),
        array(
          'icon' => '',
          'number'=> '',
          'title' => '',
        )
      ),
      'sanitize_callback' => array( 'Eduline_Repeater_Setting', 'sanitize_repeater_setting' ),
    ) 
  ) 
);

$wp_customize->add_control(
  new Eduline_Control_Repeater(
    $wp_customize,
    'eduline_counter_items',
    array(
      'section' => 'eduline_counter_section',              
      'label'   => __( 'Counter items', 'eduline-pro' ),
      'fields'  => array(
        'icon' => array(
          'type'        => 'font',
          'label'       => __( 'Font Awesome Icon', 'eduline-pro' ),
        ),
        'number' => array(
          'type'        => 'text',
          'label'       => __( 'Counter Number', 'eduline-pro' ),
        ),
        'title' => array(
          'type'        => 'text',
          'label'       => __( 'Counter Title', 'eduline-pro' ),
        )
      ),
      'row_label' => array(
        'type' => 'field',
        'value' => __( 'counter', 'eduline-pro' ),
        'field' => 'title'
      )                        
    )
  )
);


/*----------------------------------------------------------------------------------------------------------------------------------------*/
/**
 * Frontpage Service section
 *
 * @since 1.0.0
 */

$wp_customize->add_section(
  'eduline_frontpage_service_section',
  array(
    'priority'       => 3,
    'panel'          => 'eduline_frontpage_settings_panel',
    'capability'     => 'edit_theme_options',
    'theme_supports' => '',
    'title'          => __( 'Service Section', 'eduline-pro' ),
    'description'    => __( 'Managed the Service display at Frontpage section.', 'eduline-pro' ),
  )
);

//Service Enable/Disable
$wp_customize->add_setting( 'eduline_service_enable', array(
  'capability'            => 'edit_theme_options',
  'default'               => 0,
  'sanitize_callback'     => 'eduline_sanitize_checkbox'
) );

$wp_customize->add_control( 'eduline_service_enable', array(
  'label'                 =>  __( 'Enable/Disable Service section', 'eduline-pro' ),
  'section'               => 'eduline_frontpage_service_section',
  'type'                  => 'checkbox',
  'settings'              => 'eduline_service_enable',
) );

//Service title and description with featured Image
$wp_customize->add_setting( 'eduline_service_page_title', array(
  'capability'            => 'edit_theme_options',
  'default'               => '',
  'sanitize_callback'     => 'eduline_sanitize_dropdown_pages'
) );

$wp_customize->add_control( 'eduline_service_page_title', array( 
  'label'                 =>  __( 'Select Page for  Service Page title & description with Featured Image', 'eduline-pro' ),
  'section'               => 'eduline_frontpage_service_section',
  'type'                  => 'dropdown-pages',
  'settings'              => 'eduline_service_page_title',
) );

$wp_customize->add_setting('service_post_type_info', array(
  'default' => '',
  'type' => 'customtext',
  'capability' => 'edit_theme_options',
  'transport' => 'refresh',
  'sanitize_callback' => 'sanitize_text_field',
) );
$wp_customize->add_control( new Custom_Post_Type_Info( $wp_customize, 'service_post_type_info', array(
  'label' => esc_attr__( 'Go to Service Page', 'eduline-pro' ),
  'section' => 'eduline_frontpage_service_section',
  'settings' => 'service_post_type_info',
  'link'    => 'service',
  'extra' => esc_attr__( ' for more info', 'eduline-pro' )
) ) 
);


$wp_customize->add_setting( 'eduline_service_number', array(
  'capability'            => 'edit_theme_options',
  'default'               => '6',
  'sanitize_callback'     => 'absint'
));

$wp_customize->add_control( 'eduline_service_number', array(
  'label'                 =>  __( 'Number of Service to Show in Front Page', 'eduline-pro' ),
  'description'           =>  __( 'input 3,4,5,6,7,8,9,10', 'eduline-pro' ),
  'section'               => 'eduline_frontpage_service_section',
  'type'                  => 'number',
  'settings' => 'eduline_service_number',
) );
/*----------------------------------------------------------------------------------------------------------------------------------------*/
/**
 * Front page Course section
 *
 * @since 1.0.0
 */

$wp_customize->add_section(
  'eduline_frontpage_course_section',
  array(
    'priority'       => 4,
    'panel'          => 'eduline_frontpage_settings_panel',
    'capability'     => 'edit_theme_options',
    'theme_supports' => '',
    'title'          => __( 'Course Section', 'eduline-pro' ),
    'description'    => __( 'Managed the Course display at Frontpage section.', 'eduline-pro' ),
  )
);

/**
 * Enable/Disable for Courses
 *
 * @since 1.0.0
 */
$wp_customize->add_setting(
  'eduline_course_enable',
  array(
    'default'           => 0,
    'sanitize_callback' => 'eduline_sanitize_checkbox',
  )
);

$wp_customize->add_control(
  'eduline_course_enable',
  array(
    'section'     => 'eduline_frontpage_course_section',
    'label'       => __( 'Enable/Disable for  Courses', 'eduline-pro' ),
    'type'        => 'checkbox'
  )       
);


$wp_customize->add_setting( 'eduline_course_title', array(
  'capability'            => 'edit_theme_options',
  'default'               => '',
  'sanitize_callback'     => 'eduline_sanitize_dropdown_pages'
) );

$wp_customize->add_control('eduline_course_title',
  array(
    'label'                 =>  __( 'Select Page for Course title and description', 'eduline-pro' ),
    'section' => 'eduline_frontpage_course_section',
    'type'=> 'dropdown-pages',
    'settings' => 'eduline_course_title'
  )
);


$wp_customize->add_setting('lp_course_post_type_info', array(
  'default' => '',
  'type' => 'customtext',
  'capability' => 'edit_theme_options',
  'transport' => 'refresh',
  'sanitize_callback' => 'sanitize_text_field',
) );
$wp_customize->add_control( new Custom_Post_Type_Info( $wp_customize, 'lp_course_post_type_info', array(
  'label' => esc_attr__( 'Go to Learnpress >>  Courses', 'eduline-pro' ),
  'section' => 'eduline_frontpage_course_section',
  'settings' => 'lp_course_post_type_info',
  'link'    => 'lp_course',
  'extra' => esc_attr__( ' for more info', 'eduline-pro' )
) ) 
);


$wp_customize->add_setting( 'eduline_course_number', array(
  'capability'            => 'edit_theme_options',
  'default'               => '3',
  'sanitize_callback'     => 'absint'
));

$wp_customize->add_control( 'eduline_course_number', array(
  'label'                 =>  __( 'Number of Course to Show in Front Page', 'eduline-pro' ),
  'description'           =>  __( 'input 3,4,5,6,7,8,9,10', 'eduline-pro' ),
  'section'               => 'eduline_frontpage_course_section',
  'type'                  => 'number',
  'settings' => 'eduline_course_number',
) );

$wp_customize->add_setting( 'eduline_buy_course_text', array(
  'capability'            => 'edit_theme_options',
  'default'               => '',
  'sanitize_callback'     => 'sanitize_text_field'
) );


$wp_customize->add_control( 'eduline_buy_course_text', array(
 'label'                 =>  __( 'Buy Course Text', 'eduline-pro' ),
 'section'               => 'eduline_frontpage_course_section',
 'type'                  => 'text',
 'settings' => 'eduline_buy_course_text',
) );

/*----------------------------------------------------------------------------------------------------------------------------------------*/
/**
 * Front page  Registration section
 *
 * @since 1.0.0
 */

$wp_customize->add_section(
  'eduline_frontpage_registration_section',
  array(
    'priority'       => 5,
    'panel'          => 'eduline_frontpage_settings_panel',
    'capability'     => 'edit_theme_options',
    'theme_supports' => '',
    'title'          => __( 'Registration Section', 'eduline-pro' ),
    'description'    => __( 'Managed the  Registration display at Frontpage section.', 'eduline-pro' ),
  )
);

/**
 * Enable/Disable for  Registration
 *
 * @since 1.0.0
 */
$wp_customize->add_setting(
  'eduline_registration_enable',
  array(
    'default'           => 0,
    'sanitize_callback' => 'eduline_sanitize_checkbox',
  )
);

$wp_customize->add_control(
  'eduline_registration_enable',
  array(
    'section'     => 'eduline_frontpage_registration_section',
    'label'       => __( 'Enable/Disable for Registration', 'eduline-pro' ),
    'type'        => 'checkbox'
  )       
);

$wp_customize->add_setting( 'eduline_registration_title', array(
  'capability'            => 'edit_theme_options',
  'default'               => '',
  'sanitize_callback'     => 'eduline_sanitize_dropdown_pages'
) );

$wp_customize->add_control('eduline_registration_title',
  array(
    'label'                 =>  __( 'Select Page for Registration Title and Description', 'eduline-pro' ),
    'section' => 'eduline_frontpage_registration_section',
    'type'=> 'dropdown-pages',
    'settings' => 'eduline_registration_title'
  )
);

// Newsletter Form Shortcode Descriptions
$wp_customize->add_setting('eduline_newsletter_shortcode',array(
  'default'       =>      '',
  'sanitize_callback'     =>  'sanitize_text_field'
));
$wp_customize->add_control('eduline_newsletter_shortcode',array(
  'section'       =>      'eduline_frontpage_registration_section',
  'label'                 =>  __( 'News Letter Section Use Shortcode', 'eduline-pro' ),
  /* translators: %s: Description */
  'description'           => sprintf( __( 'Use Newsletter Plugins shortcode: Eg: %1$s. %2$s See more here %3$s', 'eduline-pro' ), '[newsletter_form]','<a href="'.esc_url('https://wordpress.org/plugins/newsletter/').'" target="_blank">','</a>' ),
  'type'          =>      'text',
  'settings'    =>    'eduline_newsletter_shortcode'
));
/*----------------------------------------------------------------------------------------------------------------------------------------*/
/**
 * Front page  Team section
 *
 * @since 1.0.0
 */

$wp_customize->add_section(
  'eduline_frontpage_team_section',
  array(
    'priority'       => 6,
    'panel'          => 'eduline_frontpage_settings_panel',
    'capability'     => 'edit_theme_options',
    'theme_supports' => '',
    'title'          => __( 'Team Section', 'eduline-pro' ),
    'description'    => __( 'Managed the  Team display at Frontpage section.', 'eduline-pro' ),
  )
);

/**
 * Enable/Disable for Team
 *
 * @since 1.0.0
 */
$wp_customize->add_setting(
  'eduline_team_enable',
  array(
    'default'           => 0,
    'sanitize_callback' => 'eduline_sanitize_checkbox',
  )
);

$wp_customize->add_control(
  'eduline_team_enable',
  array(
    'section'     => 'eduline_frontpage_team_section',
    'label'       => __( 'Enable/Disable for Team', 'eduline-pro' ),
    'type'        => 'checkbox'
  )       
);

//Team Title
$wp_customize->add_setting( 'eduline_team_page_title', array(
  'capability'            => 'edit_theme_options',
  'default'               => '',
  'sanitize_callback'     => 'eduline_sanitize_dropdown_pages'
) );

$wp_customize->add_control( 'eduline_team_page_title', array(
  'label'                 =>  __( 'Select Page for Team Title & Description', 'eduline-pro' ),
  'section'               => 'eduline_frontpage_team_section',
  'type'                  => 'dropdown-pages',
  'settings'              => 'eduline_team_page_title',
) );


$wp_customize->add_setting('team_post_type_info', array(
  'default' => '',
  'type' => 'customtext',
  'capability' => 'edit_theme_options',
  'transport' => 'refresh',
  'sanitize_callback' => 'sanitize_text_field',
) );
$wp_customize->add_control( new Custom_Post_Type_Info( $wp_customize, 'team_post_type_info', array(
  'label' => esc_attr__( 'Go to Team', 'eduline-pro' ),
  'section' => 'eduline_frontpage_team_section',
  'settings' => 'team_post_type_info',
  'link'    => 'team',
  'extra' => esc_attr__( ' for more info', 'eduline-pro' )
) ) 
);


$wp_customize->add_setting( 'eduline_team_number', array(
  'capability'            => 'edit_theme_options',
  'default'               => '3',
  'sanitize_callback'     => 'absint'
));

$wp_customize->add_control( 'eduline_team_number', array(
  'label'                 =>  __( 'Number of Course to Show in Front Page', 'eduline-pro' ),
  'description'           =>  __( 'input 3,4,5,6,7,8,9,10', 'eduline-pro' ),
  'section'               => 'eduline_frontpage_team_section',
  'type'                  => 'number',
  'settings' => 'eduline_team_number',
) );
/*----------------------------------------------------------------------------------------------------------------------------------------*/
/**
 * Front page Testimonials section
 *
 * @since 1.0.0
 */

$wp_customize->add_section(
  'eduline_frontpage_testimonials_section',
  array(
    'priority'       => 7,
    'panel'          => 'eduline_frontpage_settings_panel',
    'capability'     => 'edit_theme_options',
    'theme_supports' => '',
    'title'          => __( 'Testimonials Section', 'eduline-pro' ),
    'description'    => __( 'Managed the  Testimonials display at Frontpage section.', 'eduline-pro' ),
  )
);

/**
 * Enable/Disable for Testimonials
 *
 * @since 1.0.0
 */
$wp_customize->add_setting(
  'eduline_testimonials_enable',
  array(
    'default'           => 0,
    'sanitize_callback' => 'eduline_sanitize_checkbox',
  )
);

$wp_customize->add_control(
  'eduline_testimonials_enable',
  array(
    'section'     => 'eduline_frontpage_testimonials_section',
    'label'       => __( 'Enable/Disable for Testimonials', 'eduline-pro' ),
    'type'        => 'checkbox'
  )       
);


$wp_customize->add_setting( 'eduline_testimonials_title', array(
  'capability'            => 'edit_theme_options',
  'default'               => '',
  'sanitize_callback'     => 'eduline_sanitize_dropdown_pages'
) );

$wp_customize->add_control('eduline_testimonials_title',
  array(
    'label'                 =>  __( 'Select Page for Testimonials Title, Featured Image and vedio link in description', 'eduline-pro' ),
    'section' => 'eduline_frontpage_testimonials_section',
    'type'=> 'dropdown-pages',
    'settings' => 'eduline_testimonials_title'
  )
);

// Testimonials Custom Post Type
$wp_customize->add_setting('testimonial_post_type_info', array(
  'default' => '',
  'type' => 'customtext',
  'capability' => 'edit_theme_options',
  'transport' => 'refresh',
  'sanitize_callback' => 'sanitize_text_field',
) );

$wp_customize->add_control( new Custom_Post_Type_Info( $wp_customize, 'testimonial_post_type_info', array(
  'label' => esc_attr__( 'Go to Testimonial', 'eduline-pro' ),
  'section' => 'eduline_frontpage_testimonials_section',
  'settings' => 'testimonial_post_type_info',
  'link'    => 'testimonial',
  'extra' => esc_attr__( ' for more info', 'eduline-pro' )
) ) 
);

// Testimonial
$wp_customize->add_setting( 'eduline_testimonial_number', array(
  'capability'            => 'edit_theme_options',
  'default'               => '3',
  'sanitize_callback'     => 'absint'
));

$wp_customize->add_control( 'eduline_testimonial_number', array(
  'label'                 =>  __( 'Number of Testimonial to Show in Front Page', 'eduline-pro' ),
  'description'           =>  __( 'input 3,4,5,6,7,8,9,10', 'eduline-pro' ),
  'section'               => 'eduline_frontpage_testimonials_section',
  'type'                  => 'number',
  'settings' => 'eduline_testimonial_number',
));

/*----------------------------------------------------------------------------------------------------------------------------------------*/
/**
 * Front page Upcoming Events section
 *
 * @since 1.0.0
 */

$wp_customize->add_section(
  'eduline_frontpage_events_section',
  array(
    'priority'       => 8,
    'panel'          => 'eduline_frontpage_settings_panel',
    'capability'     => 'edit_theme_options',
    'theme_supports' => '',
    'title'          => __( 'Events Section', 'eduline-pro' ),
    'description'    => __( 'Managed the  Events display at Frontpage section.', 'eduline-pro' ),
  )
);

/**
 * Enable/Disable for  Events
 *
 * @since 1.0.0
 */
$wp_customize->add_setting(
  'eduline_events_enable',
  array(
    'default'           => 0,
    'sanitize_callback' => 'eduline_sanitize_checkbox',
  )
);

$wp_customize->add_control(
  'eduline_events_enable',
  array(
    'section'     => 'eduline_frontpage_events_section',
    'label'       => __( 'Enable/Disable for Events', 'eduline-pro' ),
    'type'        => 'checkbox'
  )       
);


$wp_customize->add_setting( 'eduline_events_title', array(
  'capability'            => 'edit_theme_options',
  'default'               => '',
  'sanitize_callback'     => 'eduline_sanitize_dropdown_pages'
) );

$wp_customize->add_control('eduline_events_title',
  array(
    'label'                 =>  __( 'Select Page for Events title & Description', 'eduline-pro' ),
    'section' => 'eduline_frontpage_events_section',
    'type'=> 'dropdown-pages',
    'settings' => 'eduline_events_title'
  )
);


// Events Custom Post Type
$wp_customize->add_setting('events_post_type_info', array(
  'default' => '',
  'type' => 'customtext',
  'capability' => 'edit_theme_options',
  'transport' => 'refresh',
  'sanitize_callback' => 'sanitize_text_field',
) );

$wp_customize->add_control( new Custom_Post_Type_Info( $wp_customize, 'events_post_type_info', array(
  'label' => esc_attr__( 'Go to Event', 'eduline-pro' ),
  'section' => 'eduline_frontpage_events_section',
  'settings' => 'events_post_type_info',
  'link'    => 'event',
  'extra' => esc_attr__( ' for more info', 'eduline-pro' )
) ) 
);

// Events
$wp_customize->add_setting( 'eduline_events_number', array(
  'capability'            => 'edit_theme_options',
  'default'               => '3',
  'sanitize_callback'     => 'absint'
));

$wp_customize->add_control( 'eduline_events_number', array(
  'label'                 =>  __( 'Number of Events to Show in Front Page', 'eduline-pro' ),
  'description'           =>  __( 'input 3,4,5,6,7,8,9,10', 'eduline-pro' ),
  'section'               => 'eduline_frontpage_events_section',
  'type'                  => 'number',
  'settings' => 'eduline_events_number',
));

/*----------------------------------------------------------------------------------------------------------------------------------------*/
/**
 * Front page Become A Teacher section
 *
 * @since 1.0.0
 */

$wp_customize->add_section(
  'eduline_frontpage_become_section',
  array(
    'priority'       => 9,
    'panel'          => 'eduline_frontpage_settings_panel',
    'capability'     => 'edit_theme_options',
    'theme_supports' => '',
    'title'          => __( 'Become Section', 'eduline-pro' ),
    'description'    => __( 'Managed the  Become A Teacher display at Frontpage section.', 'eduline-pro' ),
  )
);

/**
 * Enable/Disable for Become
 *
 * @since 1.0.0
 */
$wp_customize->add_setting(
  'eduline_become_enable',
  array(
    'default'           => 0,
    'sanitize_callback' => 'eduline_sanitize_checkbox',
  )
);

$wp_customize->add_control(
  'eduline_become_enable',
  array(
    'section'     => 'eduline_frontpage_become_section',
    'label'       => __( 'Enable/Disable for Become A Teacher', 'eduline-pro' ),
    'type'        => 'checkbox'
  )       
);

$wp_customize->add_setting( 'eduline_become_title', array(
  'capability'            => 'edit_theme_options',
  'default'               => '',
  'sanitize_callback'     => 'eduline_sanitize_dropdown_pages'
) );

$wp_customize->add_control('eduline_become_title',
  array(
    'label'                 =>  __( 'Select Page for Become A Teacher title & Description', 'eduline-pro' ),
    'section' => 'eduline_frontpage_become_section',
    'type'=> 'dropdown-pages',
    'settings' => 'eduline_become_title'
  )
);

$wp_customize->add_setting( 'eduline_become_button_title', array(
  'capability'            => 'edit_theme_options',
  'default'               => '',
  'sanitize_callback'     => 'sanitize_text_field'
) );

$wp_customize->add_control( 'eduline_become_button_title', array(
  'label'                 =>  __( 'Button Title For Become A Teacher', 'eduline-pro' ),
  'description'           =>  __( 'Get Started', 'eduline-pro' ),
  'section'               => 'eduline_frontpage_become_section',
  'type'                  => 'text',
  'settings' => 'eduline_become_button_title',
) );

$wp_customize->add_setting( 'eduline_become_button_url', array(
  'capability'            => 'edit_theme_options',
  'default'               => '',
  'sanitize_callback'     => 'esc_url_raw'
) );

$wp_customize->add_control( 'eduline_become_button_url', array(
  'label'                 =>  __( 'Type URL For button Title of Become A Teacher', 'eduline-pro' ),
  'description'           =>  __( '#', 'eduline-pro' ),
  'section'               => 'eduline_frontpage_become_section',
  'type'                  => 'url',
  'settings' => 'eduline_become_button_url',
) );


/*----------------------------------------------------------------------------------------------------------------------------------------*/
/**
 * Front page blog section
 *
 * @since 1.0.0
 */

$wp_customize->add_section(
  'eduline_frontpage_blog_section',
  array(
    'priority'       => 10,
    'panel'          => 'eduline_frontpage_settings_panel',
    'capability'     => 'edit_theme_options',
    'theme_supports' => '',
    'title'          => __( 'Blog Section', 'eduline-pro' ),
    'description'    => __( 'Managed the  Blog display at Frontpage section.', 'eduline-pro' ),
  )
);

/**
 * Enable/Disable for Blog Section
 *
 * @since 1.0.0
 */
$wp_customize->add_setting(
  'eduline_blog_enable',
  array(
    'default'           => 1,
    'sanitize_callback' => 'eduline_sanitize_checkbox',
  )
);

$wp_customize->add_control(
  'eduline_blog_enable',
  array(
    'section'     => 'eduline_frontpage_blog_section',
    'label'       => __( 'Enable/Disable for Blog', 'eduline-pro' ),
    'type'        => 'checkbox'
  )       
);


$wp_customize->add_setting( 'eduline_blog_title', array(
  'capability'            => 'edit_theme_options',
  'default'               => '',
  'sanitize_callback'     => 'eduline_sanitize_dropdown_pages'
) );

$wp_customize->add_control('eduline_blog_title',
  array(
    'label'                 =>  __( 'Select Page for Blog title & Description', 'eduline-pro' ),
    'section' => 'eduline_frontpage_blog_section',
    'type'=> 'dropdown-pages',
    'settings' => 'eduline_blog_title'
  )
);

//Category select for blogs
$wp_customize->add_setting('eduline_blog_category_id',array(
  'capability'        => 'edit_theme_options',
  'sanitize_callback' => 'eduline_sanitize_category',
  'default' =>  '1',
)
);

$wp_customize->add_control(new Eduline_Customize_Dropdown_Taxonomies_Control($wp_customize,'eduline_blog_category_id',
  array(
   'label' => __('Select Category for Blog','eduline-pro'),
   'section' => 'eduline_frontpage_blog_section',
   'settings' => 'eduline_blog_category_id',
   'type'=> 'dropdown-taxonomies',
 )
));

$wp_customize->add_setting( 'eduline_blog_number', array(
  'capability'            => 'edit_theme_options',
  'default'               => '3',
  'sanitize_callback'     => 'absint'
));

$wp_customize->add_control( 'eduline_blog_number', array(
  'label'                 =>  __( 'Number of Recent Blogs to Show in Front Page', 'eduline-pro' ),
  'description'           =>  __( 'input 3,4,5,6,7,8,9,10', 'eduline-pro' ),
  'section'               => 'eduline_frontpage_blog_section',
  'type'                  => 'number',
  'settings' => 'eduline_blog_number',
) );

/*----------------------------------------------------------------------------------------------------------------------------------------*/
/**
 * Front page Call to Action section
 *
 * @since 1.0.0
 */

$wp_customize->add_section(
  'eduline_frontpage_cta_section',
  array(
    'priority'       => 11,
    'panel'          => 'eduline_frontpage_settings_panel',
    'capability'     => 'edit_theme_options',
    'theme_supports' => '',
    'title'          => __( 'Call to Action Section', 'eduline-pro' ),
    'description'    => __( 'Managed the  Call to Action display at Frontpage section.', 'eduline-pro' ),
  )
);

/**
 * Enable/Disable for Become
 *
 * @since 1.0.0
 */
$wp_customize->add_setting(
  'eduline_cta_enable',
  array(
    'default'           => 0,
    'sanitize_callback' => 'eduline_sanitize_checkbox',
  )
);

$wp_customize->add_control(
  'eduline_cta_enable',
  array(
    'section'     => 'eduline_frontpage_cta_section',
    'label'       => __( 'Enable/Disable for Call to Action', 'eduline-pro' ),
    'type'        => 'checkbox'
  )       
);

$wp_customize->add_setting( 'eduline_cta_title', array(
  'capability'            => 'edit_theme_options',
  'default'               => '',
  'sanitize_callback'     => 'eduline_sanitize_dropdown_pages'
) );

$wp_customize->add_control('eduline_cta_title',
  array(
    'label'                 =>  __( 'Select Page for Call to Action title & Description', 'eduline-pro' ),
    'section' => 'eduline_frontpage_cta_section',
    'type'=> 'dropdown-pages',
    'settings' => 'eduline_cta_title'
  )
);

$wp_customize->add_setting( 'eduline_cta_button_title', array(
  'capability'            => 'edit_theme_options',
  'default'               => '',
  'sanitize_callback'     => 'sanitize_text_field'
) );

$wp_customize->add_control( 'eduline_cta_button_title', array(
  'label'                 =>  __( 'Button Title For Call to Action', 'eduline-pro' ),
  'description'           =>  __( 'Get Started', 'eduline-pro' ),
  'section'               => 'eduline_frontpage_cta_section',
  'type'                  => 'text',
  'settings' => 'eduline_cta_button_title',
) );

$wp_customize->add_setting( 'eduline_cta_button_url', array(
  'capability'            => 'edit_theme_options',
  'default'               => '',
  'sanitize_callback'     => 'esc_url_raw'
) );

$wp_customize->add_control( 'eduline_cta_button_url', array(
  'label'                 =>  __( 'Type URL For button Title of Call to Action', 'eduline-pro' ),
  'description'           =>  __( '#', 'eduline-pro' ),
  'section'               => 'eduline_frontpage_cta_section',
  'type'                  => 'url',
  'settings' => 'eduline_cta_button_url',
) );
}